#!/bin/sh /etc/rc.common
# Copyright (C) 2015 OpenWrt.org

if [ "$( which vdsl_cpe_control )" ]; then
	XDSL_CTRL=vdsl_cpe_control
else
	XDSL_CTRL=dsl_cpe_control
fi

#
# Basic functions to send CLI commands to the vdsl_cpe_control daemon
#
dsl_cmd() {
	killall -0 ${XDSL_CTRL} && (
	# Sometimes LuCI will lock the pipe for lucistat, If this happens
	# we need to kill it, otherwise our status page won't show up.
	if [ "$(pidof cat /tmp/pipe/dsl_cpe1_ack)" ] 
	then
		kill $(pidof cat /tmp/pipe/dsl_cpe1_ack)
	fi
	# Use a different pipe for lucistat to prevent further lockups.
		if [ "$action" = "lucistat" ]; then
			echo "$@" > /tmp/pipe/dsl_cpe1_cmd
			cat /tmp/pipe/dsl_cpe1_ack
		else
			echo "$@" > /tmp/pipe/dsl_cpe0_cmd
			cat /tmp/pipe/dsl_cpe0_ack
		fi
	)
}
dsl_val() {
	echo $(expr "$1" : '.*'$2'=\([-\.[:alnum:]]*\).*')
}

#
# Simple divide by 10 routine to cope with one decimal place
#
dbt() {
	local a=$(expr $1 / 10)
	local b=$(expr $1 % 10)
	
	if [ "$b" -lt 0 ]
	then b=${b#-}
	fi

	echo "${a}.${b}"
}

#
# Take a number and convert to k or meg
#
scale() {
	local val=$1
	local a
	local b

	if [ "$val" -gt 1000000 ]; then
		a=$(expr $val / 1000)
		b=$(expr $a % 1000)
		a=$(expr $a / 1000)
		printf "%d.%03d Mb" ${a} ${b}
	elif [ "$val" -gt 1000 ]; then
		a=$(expr $val / 1000)
		printf "%d Kb" ${a}
	else
		echo "${val} b"
	fi
}

scale_latency() {
	local val=$1
	local a
	local b
	a=$(expr $val / 100)
	b=$(expr $val % 100)
	printf "%d.%d ms" ${a} ${b}
}

#
# Chipset
#
chipset() {
	local vig

	vig=$(dsl_cmd vig)
	csfw=$(dsl_val "$vig" DSL_ChipSetFWVersion)
	csapi=$(dsl_val "$vig" DSL_DriverVersionApi)
	csmei=$(dsl_val "$vig" DSL_DriverVersionMeiBsp)

	if [ "$action" = "lucistat" ]; then
		echo "dsl.chipset=\"Lantiq™ XWAY™ VRX268\""
		echo "dsl.firmware_version=\"${csfw}\""
		echo "dsl.api_version=\"${csapi}\""
		echo "dsl.mei_version=\"${csmei}\""
	else
		echo "Chipset:		Lantiq™ XWAY™ VRX268"
		echo "Firmware Version:	${csfw}"
		echo "API Version:		${csapi}"
		echo "MEI Version:		${csmei}"
	fi
}

#
# Power Management Mode
#
power_mode() {
	local pmsg=$(dsl_cmd g997pmsg)
	local pm=$(dsl_val "$pmsg" nPowerManagementStatus);
	local s;
	case "$pm" in
		"-1")		s="Power management state is not available" ;;
		"0")		s="L0 - Synchronized" ;;
		"1")		s="L1 - Power Down Data transmission (G.992.2)" ;;
		"2")		s="L2 - Power Down Data transmission (G.992.3 and G.992.4)" ;;
		"3")		s="L3 - No power" ;;
		*)			s="unknown" ;;
	esac
	if [ "$action" = "lucistat" ]; then
		echo "dsl.power_mode_num=\"$pm\""
		echo "dsl.power_mode_s=\"$s\""
	else
		echo "Power Management Mode:	$s"
	fi
}

#
# Is the line up? Or what state is it in?
#
line_state() {
	local lsg=$(dsl_cmd lsg)
	local ls=$(dsl_val "$lsg" nLineState);
	local s;

	case "$ls" in
		"0x0")		s="not initialized" ;;
		"0x1")		s="exception" ;;
		"0x10")		s="not updated" ;;
		"0xff")		s="idle request" ;;
		"0x100")	s="idle" ;;
		"0x1ff")	s="silent request" ;;
		"0x200")	s="silent" ;;
		"0x300")	s="handshake" ;;
		"0x380")	s="full_init" ;;
		"0x400")	s="discovery" ;;
		"0x500")	s="training" ;;
		"0x600")	s="analysis" ;;
		"0x700")	s="exchange" ;;
		"0x800")	s="showtime_no_sync" ;;
		"0x801")	s="showtime_tc_sync" ;;
		"0x900")	s="fastretrain" ;;
		"0xa00")	s="lowpower_l2" ;;
		"0xb00")	s="loopdiagnostic active" ;;
		"0xb10")	s="loopdiagnostic data exchange" ;;
		"0xb20")	s="loopdiagnostic data request" ;;
		"0xc00")	s="loopdiagnostic complete" ;;
		"0x1000000")	s="test" ;;
		"0xd00")	s="resync" ;;
		"0x3c0")	s="short init entry" ;;
		"")		s="not running daemon"; ls="0xfff" ;;
		*)		s="unknown" ;;
	esac

	if [ "$action" = "lucistat" ]; then
		echo "dsl.line_state_num=$ls"
		echo "dsl.line_state_detail=\"$s\""
		if [ "$ls" = "0x801" ]; then
			echo "dsl.line_state=\"UP\""
		else
			echo "dsl.line_state=\"DOWN\""
		fi
	else
		if [ "$ls" = "0x801" ]; then
			echo "Line State:		UP [$ls: $s]"
		else
			echo "Line State:		DOWN [$ls: $s]"
		fi
	fi
}

#
# Work out how long the line has been up
#
line_uptime() {
	local ccsg
	local et
	local etr
	local d
	local h
	local m
	local s
	local rc=""

	ccsg=$(dsl_cmd pmccsg 0 0 0)
	et=$(dsl_val "$ccsg" nElapsedTime)

	[ -z "$et" ] && et=0

	d=$(expr $et / 86400)
	etr=$(expr $et % 86400)
	h=$(expr $etr / 3600)
	etr=$(expr $etr % 3600)
	m=$(expr $etr / 60)
	s=$(expr $etr % 60)


	[ "${d}${h}${m}${s}" -ne 0 ] && rc="${s}s"
	[ "${d}${h}${m}" -ne 0 ] && rc="${m}m ${rc}"
	[ "${d}${h}" -ne 0 ] && rc="${h}h ${rc}"
	[ "${d}" -ne 0 ] && rc="${d}d ${rc}"

	[ -z "$rc" ] && rc="down"
	if [ "$action" = "lucistat" ]; then
		echo "dsl.line_uptime=\"${rc}\""
		return
	fi
	echo "Line Uptime:		${rc}"
}

#
# Resyncs
#
resyncs() {
	local pmlict
	local pmlictresyncs
	
	pmlict=$(dsl_cmd pmlictg)
	pmlictresyncs=$(dsl_val "$pmlict" nFullInits)
	
	if [ "$action" = "lucistat" ]; then
		echo "dsl.get_resyncs=\"${pmlictresyncs}\""
	else
		echo "Resyncs:		${pmlictresyncs}"
	fi
}

#
# Vendor information
#
vendor() {
	local g997listrg
	local cabsvid
	
	g997listrg=$(dsl_cmd g997listrg 1)
	cabsvid=$(dsl_val "$g997listrg" G994VendorID)
	
	if [ "$action" = "lucistat" ]; then
		echo "dsl.cab_chipset=\"${cabsvid}\""
	else
		echo "DSLAM/MSAN VID:		${cabsvid}"
	fi
}

#
# XTSE capabilities
#
xtse() {
	local xtusesg
	local xtse1
	local xtse2
	local xtse3
	local xtse4
	local xtse5
	local xtse6
	local xtse7
	local xtse8
	local xtse_s=""
	local annex_s=""
	local line_mode_s=""
	local cmd=""
	xtusesg=$(dsl_cmd g997xtusesg)
	xtse1=$(dsl_val "$xtusesg" XTSE1)
	xtse2=$(dsl_val "$xtusesg" XTSE2)
	xtse3=$(dsl_val "$xtusesg" XTSE3)
	xtse4=$(dsl_val "$xtusesg" XTSE4)
	xtse5=$(dsl_val "$xtusesg" XTSE5)
	xtse6=$(dsl_val "$xtusesg" XTSE6)
	xtse7=$(dsl_val "$xtusesg" XTSE7)
	xtse8=$(dsl_val "$xtusesg" XTSE8)
	# Evaluate Annex (according to G.997.1, 7.3.1.1.1)
	if [ $((xtse1 & 13)) != 0 \
	-o $((xtse2 & 1)) != 0 \
	-o $((xtse3 & 12)) != 0 \
	-o $((xtse4 & 3)) != 0 \
	-o $((xtse6 & 3)) != 0 \
	-o $((xtse8 & 1)) != 0 ]; then
		annex_s=" A,"
	fi
	if [ $((xtse1 & 48)) != 0 \
	-o $((xtse2 & 2)) != 0 \
	-o $((xtse3 & 48)) != 0 \
	-o $((xtse6 & 12)) != 0 \
	-o $((xtse8 & 2)) != 0 ]; then
		annex_s="$annex_s B,"
	fi
	if [ $((xtse1 & 194)) != 0 \
	-o $((xtse2 & 12)) != 0 \
	-o $((xtse8 & 4)) != 0 ]; then
		annex_s="$annex_s C,"
	fi
	if [ $((xtse4 & 48)) != 0 \
	-o $((xtse5 & 3)) != 0 \
	-o $((xtse6 & 192)) != 0 ]; then
		annex_s="$annex_s I,"
	fi
	if [ $((xtse4 & 192)) != 0 \
	-o $((xtse7 & 3)) != 0 ]; then
		annex_s="$annex_s J,"
	fi
	if [ $((xtse5 & 60)) != 0 ]; then
		annex_s="$annex_s L,"
	fi
	if [ $((xtse5 & 192)) != 0 \
	-o $((xtse7 & 12)) != 0 ]; then
		annex_s="$annex_s M,"
	fi
	annex_s=`echo ${annex_s:1}`
	annex_s=`echo ${annex_s%?}`
	# Evaluate Line Mode (according to G.997.1, 7.3.1.1.1)
	# Regional standard: ANSI T1.413
	if [ $((xtse1 & 1)) != 0  ]; then
		line_mode_s=" T1.413,"
	fi
	# Regional standard: TS 101 388
	if [ $((xtse1 & 1)) != 0  ]; then
		line_mode_s="$line_mode_s TS 101 388,"
	fi
	if [ $((xtse1 & 252)) != 0  ]; then
		line_mode_s="$line_mode_s G.992.1 (ADSL),"
	fi
	if [ $((xtse2 & 15)) != 0  ]; then
		line_mode_s="$line_mode_s G.992.2 (ADSL lite),"
	fi
	if [ $((xtse3 & 60)) != 0 \
	-o $((xtse4 & 240)) != 0 \
	-o $((xtse5 & 252)) != 0  ]; then
		line_mode_s="$line_mode_s G.992.3 (ADSL2),"
	fi
	if [ $((xtse4 & 3)) != 0 \
	-o $((xtse5 & 3)) != 0  ]; then
		line_mode_s="$line_mode_s G.992.4 (ADSL2 lite),"
	fi
	if [ $((xtse6 & 199)) != 0 \
	-o $((xtse7 & 15)) != 0  ]; then
		line_mode_s="$line_mode_s G.992.5 (ADSL2+),"
	fi
	if [ $((xtse8 & 7)) != 0  ]; then
		line_mode_s="$line_mode_s G.993.2 (VDSL2),"
	fi
	#!!! PROPRIETARY & INTERMEDIATE USE !!!
	if [ $((xtse8 & 128)) != 0  ]; then
		line_mode_s="$line_mode_s G.993.1 (VDSL),"
	fi
	line_mode_s=`echo ${line_mode_s:1}`
	line_mode_s=`echo ${line_mode_s%?}`
	xtse_s="${xtse1}, ${xtse2}, ${xtse3}, ${xtse4}, ${xtse5}, ${xtse6}, ${xtse7}, ${xtse8}"
	if [ "$action" = "lucistat" ]; then
		echo "dsl.xtse1=\"$xtse1\""
		echo "dsl.xtse2=\"$xtse2\""
		echo "dsl.xtse3=\"$xtse3\""
		echo "dsl.xtse4=\"$xtse4\""
		echo "dsl.xtse5=\"$xtse5\""
		echo "dsl.xtse6=\"$xtse6\""
		echo "dsl.xtse7=\"$xtse7\""
		echo "dsl.xtse8=\"$xtse8\""
		echo "dsl.xtse_s=\"$xtse_s\""
		echo "dsl.annex_s=\"${annex_s}\""
		echo "dsl.line_mode_s=\"${line_mode_s}\""
	else
		echo "XTSE Capabilities:	${xtse_s}"
		echo "Annex:			${annex_s}"
		echo "Line Mode:		${line_mode_s}"
	fi
}

#
# Profile
#
profile() {
	local bpstg=$(dsl_cmd bpstg)
	local bp=$(dsl_val "$bpstg" nProfile);
	local s;

	case "$bp" in
		"0")		s="8a" ;;
		"1")		s="8b" ;;
		"2")		s="8c" ;;
		"3")		s="8d" ;;
		"4")		s="12a" ;;
		"5")		s="12b" ;;
		"6")		s="17a" ;;
		"7")		s="30a" ;;
		"8")		s="17b" ;;
		*)			s="unknown" ;;
	esac

	if [ "$action" = "lucistat" ]; then
		echo "dsl.line_profile=\"$s\""
	else
			echo "Profile:		$s"
	fi
}

#
# Line Features
#
line_features() {
	local lfsg
	local trellisd
	local trellisu
	local bitswapd
	local bitswapu
	local retxd
	local retxu
	local vnsd
	local vnsu
	
	lfsg=$(dsl_cmd lfsg 1)
	trellisd=$(dsl_val "$lfsg" bTrellisEnable)
	bitswapd=$(dsl_val "$lfsg" bBitswapEnable)
	retxd=$(dsl_val "$lfsg" bReTxEnable)
	vnsd=$(dsl_val "$lfsg" bVirtualNoiseSupport)
	
	lfsg=$(dsl_cmd lfsg 0)
	trellisu=$(dsl_val "$lfsg" bTrellisEnable)
	bitswapu=$(dsl_val "$lfsg" bBitswapEnable)
	retxu=$(dsl_val "$lfsg" bReTxEnable)
	vnsu=$(dsl_val "$lfsg" bVirtualNoiseSupport)
	
	case "$trellisd" in
		"0")		trellisd="D: OFF" ;;
		"1")		trellisd="D: ON" ;;
	esac

	case "$bitswapd" in
		"0")		bitswapd="D: OFF" ;;
		"1")		bitswapd="D: ON" ;;
	esac

	case "$retxd" in
		"0")		retxd="D: Not Enabled" ;;
		"1")		retxd="D: Enabled" ;;
	esac

	case "$vnsd" in
		"0")		vnsd="D: Not Supported" ;;
		"1")		vnsd="D: Supported" ;;
	esac
	case "$trellisu" in
		"0")		trellisu="U: OFF" ;;
		"1")		trellisu="U: ON" ;;
	esac

	case "$bitswapu" in
		"0")		bitswapu="U: OFF" ;;
		"1")		bitswapu="U: ON" ;;
	esac

	case "$retxu" in
		"0")		retxu="U: Not Enabled" ;;
		"1")		retxu="U: Enabled" ;;
	esac

	case "$vnsu" in
		"0")		vnsu="U: Not Supported" ;;
		"1")		vnsu="U: Supported" ;;
	esac
	
	if [ "$action" = "lucistat" ]; then
		echo "dsl.trellis_down=\"${trellisd}\""
		echo "dsl.trellis_up=\"${trellisu}\""
		echo "dsl.bitswap_down=\"${bitswapd}\""
		echo "dsl.bitswap_up=\"${bitswapu}\""
		echo "dsl.retx_down=\"${retxd}\""
		echo "dsl.retx_up=\"${retxu}\""
		echo "dsl.vns_down=\"${vnsd}\""
		echo "dsl.vns_up=\"${vnsu}\""
	else
		echo "Trellis:		${trellisd} / ${trellisu}"
		echo "Bitswap:		${bitswapd} / ${bitswapu}"
		echo "G.INP:			${retxd} / ${retxu}"
		echo "Virtual Noise Support:	${vnsd} / ${vnsu}"
	fi
}

#
# Read the data rates for both directions
#
attainable_data_rates() {
	local lsg
	local drd
	local dru
	local sdrd
	local sdru
	
	lsg=$(dsl_cmd g997lsg 1 1)
	drd=$(dsl_val "$lsg" ATTNDR)
	
	lsg=$(dsl_cmd g997lsg 0 1)
	dru=$(dsl_val "$lsg" ATTNDR)
	
	[ -z "$drd" ] && drd=0
	[ -z "$dru" ] && dru=0
	
	sdrd=$(scale $drd)
	sdru=$(scale $dru)
	
	if [ "$action" = "lucistat" ]; then
		echo "dsl.attain_data_rate_down=\"$drd\""
		echo "dsl.attain_data_rate_up=\"$dru\""
		echo "dsl.attain_data_rate_down_s=\"$sdrd\""
		echo "dsl.attain_data_rate_up_s=\"$sdru\""
	else
		echo "Attain Data Rate:	${sdrd}/s / ${sdru}/s"
	fi
}

data_rates() {
	local csg
	local drd
	local dru
	local sdrd
	local sdru
	local inpd
	local inpu

	local idd
	local sidd
	local idu
	local sidu
	
	csg=$(dsl_cmd g997csg 0 1)
	drd=$(dsl_val "$csg" ActualDataRate)
	inpd=$(dsl_val "$csg" ActualImpulseNoiseProtection)
	idd=$(dsl_val "$csg" ActualInterleaveDelay)

	csg=$(dsl_cmd g997csg 0 0)
	dru=$(dsl_val "$csg" ActualDataRate)
	inpu=$(dsl_val "$csg" ActualImpulseNoiseProtection)
	idu=$(dsl_val "$csg" ActualInterleaveDelay)

	[ -z "$drd" ] && drd=0
	[ -z "$dru" ] && dru=0

	[ -z "$idd" ] && idd=0
	[ -z "$idu" ] && idu=0

	sdrd=$(scale $drd)
	sdru=$(scale $dru)
	
	inpd=$(dbt $inpd)
	inpu=$(dbt $inpu)
	
	sidu=$(scale_latency $idu)
	sidd=$(scale_latency $idd)

	if [ "$action" = "lucistat" ]; then
		echo "dsl.data_rate_down=\"$drd\""
		echo "dsl.data_rate_up=\"$dru\""
		echo "dsl.data_rate_down_s=\"$sdrd\""
		echo "dsl.data_rate_up_s=\"$sdru\""
		echo "dsl.actual_impulse_noise_protection_down=\"$inpd\""
		echo "dsl.actual_impulse_noise_protection_up=\"$inpu\""
		echo "dsl.actual_interleave_delay_down=\"$sidd\""
		echo "dsl.actual_interleave_delay_up=\"$sidu\""
	else
		echo "Actual Data Rate:	${sdrd}/s / ${sdru}/s"
		echo "Impulse Noise Prot:	${inpd} sym / ${inpu} sym"
		echo "Interleave Delay:	${sidd} / ${sidu}"
	fi
}

interleave() {
	local fpsg
	local nfecd
	local nfecu
	local rfecd
	local rfecu
	local lsymbd
	local lsymbu
	local intlvdd
	local intlvdu
	local intlvbd
	local intlvbu
	local lpathd
	local lpathu
	
	fpsg=$(dsl_cmd g997fpsg 0 1)
	nfecd=$(dsl_val "$fpsg" nNFEC)
	rfecd=$(dsl_val "$fpsg" nRFEC)
	lsymbd=$(dsl_val "$fpsg" nLSYMB)
	intlvdd=$(dsl_val "$fpsg" nINTLVDEPTH)
	intlvbd=$(dsl_val "$fpsg" nINTLVBLOCK)
	lpathd=$(dsl_val "$fpsg" nLPATH)
	
	fpsg=$(dsl_cmd g997fpsg 0 0)
	nfecu=$(dsl_val "$fpsg" nNFEC)
	rfecu=$(dsl_val "$fpsg" nRFEC)
	lsymbu=$(dsl_val "$fpsg" nLSYMB)
	intlvdu=$(dsl_val "$fpsg" nINTLVDEPTH)
	intlvbu=$(dsl_val "$fpsg" nINTLVBLOCK)
	lpathu=$(dsl_val "$fpsg" nLPATH)
	
	if [ "$action" = "lucistat" ]; then
		echo "dsl.nfec_down=\"${nfecd}\""
		echo "dsl.nfec_up=\"${nfecu}\""
		echo "dsl.rfec_down=\"${rfecd}\""
		echo "dsl.rfec_up=\"${rfecu}\""
		echo "dsl.lsymb_down=\"${lsymbd}\""
		echo "dsl.lsymb_up=\"${lsymbu}\""
		echo "dsl.intlv_depth_down=\"${intlvdd}\""
		echo "dsl.intlv_depth_up=\"${intlvdu}\""
		echo "dsl.intlv_block_down=\"${intlvbd}\""
		echo "dsl.intlv_block_up=\"${intlvbu}\""
		echo "dsl.lpath_down=\"${lpathd}\""
		echo "dsl.lpath_up=\"${lpathu}\""
	else
		echo "NFEC:			${nfecd} / ${nfecu}"
		echo "RFEC:			${rfecd} / ${rfecu}"
		echo "LSYMB:			${lsymbd} / ${lsymbu}"
		echo "Interleave Depth:	${intlvdd} / ${intlvdu}"
		echo "Interleave Block:	${intlvbd} / ${intlvbu}"
		echo "LPATH:			${lpathd} / ${lpathu}"
	fi
}

#
# Get noise and attenuation figures
#
line_data() {
	local lsg
	local latnd
	local latnu
	local satnd
	local satnu
	local snrd
	local snru
	local actatpd
	local actatpu

	lsg=$(dsl_cmd g997lsg 1 1)
	latnd=$(dsl_val "$lsg" LATN)
	satnd=$(dsl_val "$lsg" SATN)
	snrd=$(dsl_val "$lsg" SNR)
	actatpu=$(dsl_val "$lsg" ACTATP)

	lsg=$(dsl_cmd g997lsg 0 1)
	latnu=$(dsl_val "$lsg" LATN)
	satnu=$(dsl_val "$lsg" SATN)
	snru=$(dsl_val "$lsg" SNR)
	actatpd=$(dsl_val "$lsg" ACTATP)

	[ -z "$latnd" ] && latnd=0
	[ -z "$latnu" ] && latnu=0
	[ -z "$satnd" ] && satnd=0
	[ -z "$satnu" ] && satnu=0
	[ -z "$snrd" ] && snrd=0
	[ -z "$snru" ] && snru=0
	[ -z "$actatpd" ] && actatpd=0
	[ -z "$actatpu" ] && actatpu=0

	latnd=$(dbt $latnd)
	latnu=$(dbt $latnu)
	satnd=$(dbt $satnd)
	satnu=$(dbt $satnu)
	snrd=$(dbt $snrd)
	snru=$(dbt $snru)
	actatpd=$(dbt $actatpd)
	actatpu=$(dbt $actatpu)

	if [ "$action" = "lucistat" ]; then
		echo "dsl.line_attenuation_down=\"$latnd\""
		echo "dsl.line_attenuation_up=\"$latnu\""
		echo "dsl.signal_attenuation=\"$satnd\""
		echo "dsl.signal_attenuation_up=\"$satnu\""
		echo "dsl.noise_margin_down=\"$snrd\""
		echo "dsl.noise_margin_up=\"$snru\""
		echo "dsl.transmit_power_down=\"$actatpd\""
		echo "dsl.transmit_power_up=\"$actatpu\""
	else
		echo "Line Attenuation:	${latnd}dB / ${latnu}dB"
		echo "Signal Attenuation:	${satnd}dB / ${satnu}dB"
		echo "Noise Margin:		${snrd}dB / ${snru}dB"
		echo "Transmit power:		${actatpd}dBm / ${actatpu}dBm"
	fi
}

errors() {
	local lsctg
	local dpctg
	local ccsg
	local fecd
	local fecu
	local esd
	local esu
	local sesd
	local sesu
	local lossd
	local lossu
	local uasd
	local uasu
	local crc_pd
	local crc_pu
	local crcp_pd
	local crcp_pu
	local hecd
	local hecu
	
	ccsg=$(dsl_cmd pmccsg 0 0 0)
	fecd=$(dsl_val "$ccsg" nFEC)
	ccsg=$(dsl_cmd pmccsg 0 1 0)
	fecu=$(dsl_val "$ccsg" nFEC)
	
	lsctg=$(dsl_cmd pmlsctg 0)
	esd=$(dsl_val "$lsctg" nES)
	sesd=$(dsl_val "$lsctg" nSES)
	lossd=$(dsl_val "$lsctg" nLOSS)
	uasd=$(dsl_val "$lsctg" nUAS)
	lsctg=$(dsl_cmd pmlsctg 1)
	esu=$(dsl_val "$lsctg" nES)
	sesu=$(dsl_val "$lsctg" nSES)
	lossu=$(dsl_val "$lsctg" nLOSS)
	uasu=$(dsl_val "$lsctg" nUAS)

	dpctg=$(dsl_cmd pmdpctg 0 0)
	hecd=$(dsl_val "$dpctg" nHEC)
	crc_pd=$(dsl_val "$dpctg" nCRC_P)
	crcp_pd=$(dsl_val "$dpctg" nCRCP_P)
	dpctg=$(dsl_cmd pmdpctg 0 1)
	hecu=$(dsl_val "$dpctg" nHEC)
	crc_pu=$(dsl_val "$dpctg" nCRC_P)
	crcp_pu=$(dsl_val "$dpctg" nCRCP_P)

	if [ "$action" = "lucistat" ]; then
		echo "dsl.errors_fec_down=$fecd"
		echo "dsl.errors_fec_up=$fecu"
		echo "dsl.errors_es_down=$esd"
		echo "dsl.errors_es_up=$esu"
		echo "dsl.errors_ses_down=$sesd"
		echo "dsl.errors_ses_up=$sesu"
		echo "dsl.errors_loss_down=$lossd"
		echo "dsl.errors_loss_up=$lossu"
		echo "dsl.errors_uas_down=$uasd"
		echo "dsl.errors_uas_up=$uasu"
		echo "dsl.errors_hec_down=$hecd"
		echo "dsl.errors_hec_up=$hecu"
		echo "dsl.errors_crc_p_down=$crc_pd"
		echo "dsl.errors_crc_p_up=$crc_pu"
		echo "dsl.errors_crcp_p_down=$crcp_pd"
		echo "dsl.errors_crcp_p_up=$crcp_pu"
	else
		echo "FECS:			${fecd} / ${fecu}"
		echo "ES:			${esd} / ${esu}"
		echo "SES:			${sesd} / ${sesu}"
		echo "LOSS:			${lossd} / ${lossu}"
		echo "UAS:			${uasd} / ${uasu}"
		echo "HEC:			${hecd} / ${hecu}"
		echo "CRC_P:			${crcp_pd} / ${crcp_pu}"
		echo "CRCP_P:			${crcp_pd} / ${crcp_pu}"
 	fi
 }

errors_15m() {
	local cc15mg
	local cvd
	local cvu
	local fecd
	local fecu
	
	cc15mg=$(dsl_cmd pmcc15mg 0 0 0)
	cvd=$(dsl_val "$cc15mg" nCodeViolations)
	fecd=$(dsl_val "$cc15mg" nFEC)
	
	cc15mg=$(dsl_cmd pmcc15mg 0 1 0)
	cvu=$(dsl_val "$cc15mg" nCodeViolations)
	fecu=$(dsl_val "$cc15mg" nFEC)
	
	if [ "$action" = "lucistat" ]; then
		echo "dsl.fifteenm_cv_down=\"${cvd}\""
		echo "dsl.fifteenm_cv_up=\"${cvu}\""
		echo "dsl.fifteenm_fec_down=\"${fecd}\""
		echo "dsl.fifteenm_fec_up=\"${fecu}\""
	else
		echo "15m Code Violations:	${cvd} / ${cvu}"
		echo "15m FEC Errors:		${fecd} / ${fecu}"
	fi
}

errors_1d() {
	local cc1dg
	local cvd
	local cvu
	local fecd
	local fecu
	
	cc1dg=$(dsl_cmd pmcc1dg 0 0 0)
	cvd=$(dsl_val "$cc1dg" nCodeViolations)
	fecd=$(dsl_val "$cc1dg" nFEC)
	
	cc1dg=$(dsl_cmd pmcc1dg 0 1 0)
	cvu=$(dsl_val "$cc1dg" nCodeViolations)
	fecu=$(dsl_val "$cc1dg" nFEC)
	
	if [ "$action" = "lucistat" ]; then
		echo "dsl.oneday_cv_down=\"${cvd}\""
		echo "dsl.oneday_cv_up=\"${cvu}\""
		echo "dsl.oneday_fec_down=\"${fecd}\""
		echo "dsl.oneday_fec_up=\"${fecu}\""
	else
		echo "1d Code Violations:	${cvd} / ${cvu}"
		echo "1d FEC Errors:		${fecd} / ${fecu}"
	fi
}
 
status() {
	chipset
	power_mode
	line_state
	line_uptime
	resyncs
	vendor
	xtse
	profile
	line_features
	attainable_data_rates
	data_rates
	interleave
	line_data
	errors
	errors_15m
	errors_1d
}

lucistat() {
	echo "local dsl={}"
	status
	echo "return dsl"
}